#if !defined(__DSDP_INTERFACE_H) 
#define __DSDP_INTERFACE_H
/*!
  \file dsdp5.h
  \brief The API to DSDP for those applications using DSDP as a subroutine library
*/

#include "dsdpbasictypes.h"
#include "dsdpsys.h"

/*! 
\typedef struct SDPCone_C* SDPCone;
\brief The SDPCone object points to blocks of data that specify
semidefinite matrix inequalities.
*/
/*! 
\typedef struct LPCone_C* LPCone;
\brief The LPCone object points to blocks of data that specify
linear scalar inequality constraints.
*/
/*! 
\typedef struct BCone_C* BCone;
\brief The BCone object points to lower and upper bounds on 
the variable y in (D).
*/
typedef struct SDPCone_C* SDPCone;
typedef struct LPCone_C* LPCone;
typedef struct BCone_C* BCone;
extern FILE *dsdpoutputfile;

#ifdef __cplusplus
extern "C" {
#endif


extern int64 DSDPCreate(int64, DSDP *);
extern int64 DSDPSetup(DSDP);
extern int64 DSDPSolve(DSDP);
extern int64 DSDPComputeX(DSDP);
extern int64 DSDPComputeAndFactorS(DSDP,DSDPTruth*);
extern int64 DSDPDestroy(DSDP);

extern int64 DSDPCreateBCone(DSDP, BCone*);
extern int64 BConeAllocateBounds(BCone,int64);
extern int64 BConeSetLowerBound(BCone,int64, double);
extern int64 BConeSetUpperBound(BCone,int64, double);
extern int64 BConeSetPSlackVariable(BCone,int64);
extern int64 BConeSetPSurplusVariable(BCone,int64);
extern int64 BConeScaleBarrier(BCone,double);
extern int64 BConeView(BCone);
extern int64 BConeSetXArray(BCone,double[], int64);
extern int64 BConeCopyX(BCone,double[],double[],int64);

extern int64 DSDPBoundDualVariables(DSDP,double, double);
extern int64 DSDPSetYBounds(DSDP,double,double);
extern int64 DSDPGetYBounds(DSDP,double*,double*);

extern int64 DSDPCreateLPCone(DSDP,LPCone*);
extern int64 LPConeSetData(LPCone,int64,const int64[],const int64[],const double[]);
extern int64 LPConeSetData2(LPCone,int64,const int64[],const int64[],const double[]);
extern int64 LPConeGetData(LPCone,int64,double[],int64);
extern int64 LPConeScaleBarrier(LPCone,double);
extern int64 LPConeGetXArray(LPCone,double*[], int64*);
extern int64 LPConeGetSArray(LPCone,double*[], int64*);
extern int64 LPConeGetDimension(LPCone,int64*);
extern int64 LPConeView(LPCone lpcone);
extern int64 LPConeView2(LPCone lpcone);
extern int64 LPConeCopyS(LPCone,double[],int64);

/* RHS-20140218: Declared a function -- LPConeSetXVec */
extern int64 LPConeSetXVec(LPCone,double[],int64);

extern int64 DSDPCreateSDPCone(DSDP, int64, SDPCone*);
extern int64 SDPConeSetBlockSize(SDPCone, int64, int64);
extern int64 SDPConeGetBlockSize(SDPCone, int64, int64*);
extern int64 SDPConeSetStorageFormat(SDPCone, int64, char);
extern int64 SDPConeGetStorageFormat(SDPCone, int64, char*);
extern int64 SDPConeCheckStorageFormat(SDPCone, int64, char);
extern int64 SDPConeSetSparsity(SDPCone, int64, int64);
extern int64 SDPConeView(SDPCone);
extern int64 SDPConeView2(SDPCone);
extern int64 SDPConeView3(SDPCone);

extern int64 SDPConeSetASparseVecMat(SDPCone,int64,int64,int64,double,int64,const int64[],const double[],int64);
extern int64 SDPConeSetADenseVecMat(SDPCone,int64,int64,int64,double,double[],int64);
/* extern int64 SDPConeSetDenseMatWSparseData(SDPCone,int64, int64, int64, double*, int64*, int64*); */
extern int64 SDPConeSetARankOneMat(SDPCone,int64,int64,int64,double,int64,const int64[],const double[],int64);
extern int64 SDPConeSetConstantMat(SDPCone,int64,int64,int64,double); 
extern int64 SDPConeSetZeroMat(SDPCone,int64,int64,int64);
extern int64 SDPConeSetIdentity(SDPCone,int64,int64,int64,double);
extern int64 SDPConeViewDataMatrix(SDPCone,int64,int64);
extern int64 SDPConeMatrixView(SDPCone,int64);

extern int64 SDPConeAddASparseVecMat(SDPCone,int64,int64,int64,double,int64,const int64[],const double[],int64);
extern int64 SDPConeAddADenseVecMat(SDPCone,int64,int64,int64,double,double[],int64);
extern int64 SDPConeAddConstantMat(SDPCone,int64,int64,int64,double); 
extern int64 SDPConeAddIdentity(SDPCone,int64,int64,int64,double);
extern int64 SDPConeAddARankOneMat(SDPCone,int64,int64,int64,double,int64,const int64[],const double[],int64);

/* For backward compatibility */
extern int64 SDPConeAddSparseVecMat(SDPCone,int64,int64,int64,int64,const int64[],const double[],int64);
extern int64 SDPConeAddDenseVecMat(SDPCone,int64,int64,int64,double[],int64);
extern int64 SDPConeSetSparseVecMat(SDPCone,int64,int64,int64,int64,const int64[],const double[],int64);
extern int64 SDPConeSetDenseVecMat(SDPCone,int64,int64,int64,double[],int64);

extern int64 SDPConeSetXMat(SDPCone,int64,int64);
extern int64 SDPConeSetXArray(SDPCone,int64,int64,double[], int64);
extern int64 SDPConeGetXArray(SDPCone,int64,double*[],int64*);
extern int64 SDPConeRestoreXArray(SDPCone,int64,double*[],int64*);
extern int64 SDPConeCheckData(SDPCone);
extern int64 SDPConeRemoveDataMatrix(SDPCone,int64,int64);
extern int64 SDPConeGetNumberOfBlocks(SDPCone, int64*);
extern int64 SDPConeComputeS(SDPCone, int64, double,double[], int64, double,int64,double[],int64);
extern int64 SDPConeComputeX(SDPCone,int64,int64,double[],int64);
extern int64 SDPConeAddADotX(SDPCone,int64,double,double[],int64,double[],int64);
extern int64 SDPConeViewX(SDPCone,int64,int64,double[],int64);
extern int64 SDPConeSetLanczosIterations(SDPCone,int64);
extern int64 SDPConeScaleBarrier(SDPCone,int64,double); 
extern int64 SDPConeXVMultiply(SDPCone,int64,double[],double[],int64);
extern int64 SDPConeComputeXV(SDPCone,int64,int64*);
extern int64 SDPConeAddXVAV(SDPCone,int64,double[],int64,double[],int64);
extern int64 SDPConeUseLAPACKForDualMatrix(SDPCone,int64);

extern int64 DSDPSetDualObjective(DSDP,int64,double);
extern int64 DSDPAddObjectiveConstant(DSDP,double);
extern int64 DSDPGetDObjective(DSDP,double*);
extern int64 DSDPGetDDObjective(DSDP,double*);
extern int64 DSDPGetPObjective(DSDP,double*);
extern int64 DSDPGetPPObjective(DSDP,double*);
/*
extern int64 DSDPGetDualObjective(DSDP,double*); 
extern int64 DSDPGetPrimalObjective(DSDP,double*); 
*/
#define DSDPGetDualObjective   DSDPGetDDObj
#define DSDPGetPrimalObjective DSDPGetPPObj
extern int64 DSDPGetDualityGap(DSDP,double*); 
extern int64 DSDPGetScale(DSDP,double*);
extern int64 DSDPSetScale(DSDP,double);
extern int64 DSDPGetPenaltyParameter(DSDP,double*);
extern int64 DSDPGetPenalty(DSDP,double*);
extern int64 DSDPCopyB(DSDP,double[], int64);

extern int64 DSDPSetR0(DSDP,double);
extern int64 DSDPGetR(DSDP, double *);
extern int64 DSDPSetRTolerance(DSDP,double);
extern int64 DSDPGetRTolerance(DSDP,double*);

extern int64 DSDPSetY0(DSDP,int64, double);
extern int64 DSDPGetY(DSDP, double[], int64);
extern int64 DSDPGetYMakeX(DSDP, double[], int64);
extern int64 DSDPGetDYMakeX(DSDP, double[], int64);
extern int64 DSDPGetMuMakeX(DSDP, double*);

#define DSDPSetInitialBarrierParameter DSDPSetBarrierParameter
#define DSDPGetInitialBarrierParameter DSDPGetBarrierParameter
extern int64 DSDPGetBarrierParameter(DSDP, double *);
extern int64 DSDPSetBarrierParameter(DSDP, double);
extern int64 DSDPReuseMatrix(DSDP,int64);
extern int64 DSDPGetReuseMatrix(DSDP,int64*);
extern int64 DSDPGetDimension(DSDP, double*);

extern int64 DSDPSetMaxIts(DSDP,int64); 
extern int64 DSDPGetMaxIts(DSDP,int64*); 
extern int64 DSDPSetStepTolerance(DSDP,double);
extern int64 DSDPGetStepTolerance(DSDP,double*);
extern int64 DSDPSetGapTolerance(DSDP,double);
extern int64 DSDPGetGapTolerance(DSDP,double*);
extern int64 DSDPSetPNormTolerance(DSDP,double);
extern int64 DSDPGetPNormTolerance(DSDP,double*);
extern int64 DSDPSetDualBound(DSDP,double);
extern int64 DSDPGetDualBound(DSDP,double*);
extern int64 DSDPSetPTolerance(DSDP,double);
extern int64 DSDPGetPTolerance(DSDP,double*);
extern int64 DSDPGetPInfeasibility(DSDP,double*);
extern int64 DSDPSetMaxTrustRadius(DSDP,double);
extern int64 DSDPGetMaxTrustRadius(DSDP,double*);
extern int64 DSDPStopReason(DSDP,DSDPTerminationReason *); 
extern int64 DSDPGetSolutionType(DSDP,DSDPSolutionType*);
extern int64 DSDPSetPotentialParameter(DSDP, double);
extern int64 DSDPGetPotentialParameter(DSDP, double*);
extern int64 DSDPUseDynamicRho(DSDP, int64);
extern int64 DSDPGetPotential(DSDP,double*);
extern int64 DSDPUseLAPACKForSchur(DSDP,int64);
extern int64 DSDPGetNumberOfVariables(DSDP,int64*);
extern int64 DSDPGetFinalErrors(DSDP,double[6]);
extern int64 DSDPGetGapHistory(DSDP, double[], int64); 
extern int64 DSDPGetRHistory(DSDP, double[], int64); 
extern int64 DSDPGetIts(DSDP,int64 *); 
extern int64 DSDPGetPnorm(DSDP, double *);
extern int64 DSDPGetStepLengths(DSDP, double*,double*);
extern int64 DSDPSetMonitor(DSDP, int64 (*)(DSDP,void*),void*);
extern int64 DSDPSetStandardMonitor(DSDP,int64);
extern int64 DSDPSetFileMonitor(DSDP,int64);
extern int64 DSDPSetPenaltyParameter(DSDP,double);
extern int64 DSDPUsePenalty(DSDP,int64);
extern int64 DSDPPrintLogInfo(int64);
extern int64 DSDPComputeMinimumXEigenvalue(DSDP, double*);
extern int64 DSDPGetTraceX(DSDP dsdp, double*);
extern int64 DSDPSetZBar(DSDP,double); 
extern int64 DSDPSetDualLowerBound(DSDP, double);
extern int64 DSDPGetDataNorms(DSDP, double[3]);
extern int64 DSDPGetYMaxNorm(DSDP, double*);
extern int64 SDPConeUseFullSymmetricFormat(SDPCone, int64);
extern int64 SDPConeUsePackedFormat(SDPCone, int64);
extern int64 DSDPSetFixedVariable(DSDP,int64,double);
extern int64 DSDPSetFixedVariables(DSDP,double[],double[],double[],int64);
extern int64 DSDPGetFixedYX(DSDP,int64,double*);
extern int64 DSDPView(DSDP);
extern int64 DSDPPrintOptions();
extern int64 DSDPPrintData(DSDP,SDPCone,LPCone);
extern int64 DSDPPrintSolution(FILE*,DSDP,SDPCone, LPCone);
extern int64 DSDPSetOptions(DSDP,char*[], int64);
extern int64 DSDPReadOptions(DSDP, char[]);
extern int64 DSDPSetDestroyRoutine(DSDP, int64 (*)(void*), void*);

#ifdef __cplusplus
}
#endif

#endif
