#if !defined(__DSDP_DATAMATRIXOPS_H) 
#define __DSDP_DATAMATRIXOPS_H

#include "sdpconevec.h"
#include "dsdpbasictypes.h"
/*! \file dsdpdatamat.h
\brief The interface between the SDPCone and the data matrices
 */

/*!
struct DSDPDataMat_C { void* matdata; struct DSDPDataMat_Ops* dsdpops; };

\brief Symmetric data matrix for one block in the semidefinite cone.
\sa DSDPDataMat
*/
struct DSDPDataMat_C{
  void* matdata;
  struct DSDPDataMat_Ops* dsdpops;
};

/*!
\typedef struct DSDPDataMat   DSDPDataMat;
\brief Represents a single symmetric
data matrix for one block in this semidefinite cone.
*/
typedef struct DSDPDataMat_C   DSDPDataMat;

#ifdef __cplusplus
extern "C" {
#endif

extern int64 DSDPDataMatSetData(DSDPDataMat*, struct DSDPDataMat_Ops*,  void*);
extern int64 DSDPDataMatInitialize(DSDPDataMat*);
extern int64 DSDPDataMatGetType(DSDPDataMat, int64 *);
extern int64 DSDPDataMatTest(DSDPDataMat);

extern int64 DSDPDataMatVecVec(DSDPDataMat,SDPConeVec,double*);
extern int64 DSDPDataMatDot(DSDPDataMat,double[], int64,int64,double*);
extern int64 DSDPDataMatGetRowNonzeros(DSDPDataMat, int64, int64, int64*, int64*);
extern int64 DSDPDataMatCountNonzeros(DSDPDataMat,int64*,int64);
extern int64 DSDPDataMatFNorm2(DSDPDataMat,int64,double*);
extern int64 DSDPDataMatMultiply(DSDPDataMat,SDPConeVec,SDPConeVec);
extern int64 DSDPDataMatView(DSDPDataMat);
extern int64 DSDPDataMatDestroy(DSDPDataMat*);
extern int64 DSDPDataMatGetRank(DSDPDataMat, int64*,int64);
extern int64 DSDPDataMatGetEig(DSDPDataMat, int64, SDPConeVec, DSDPIndex, double *);
extern int64 DSDPDataMatFactor(DSDPDataMat,SDPConeVec, double[],int64,double[],int64,int64[],int64);
extern int64 DSDPDataMatAddMultiple(DSDPDataMat, double, double[],int64,int64);
extern int64 DSDPDataMatAddRowMultipleToVector(DSDPDataMat, int64, double, SDPConeVec);

#ifdef __cplusplus
}
#endif

#endif


