#if !defined(__DSDP_DATAMATRIX_H) 
#define __DSDP_DATAMATRIX_H
#include "dsdpbasictypes.h"
/*! 
\file dsdpdatamat_impl.h
\brief Structure of function pointers that each SDP data matrix type 
(sparse, dense, constant, identity, ...) must implement.
*/

/* DSDP Data Matrices have particular operations, and several implementations */
/*!
struct DSDPDataMat_Ops
\brief Table of function pointers that operate on the data matrix.
*/
struct  DSDPDataMat_Ops{
  int64 id;
  int64 (*mataddallmultiple)(void*,double,double[],int64,int64);
  int64 (*matdot)(void*, double[], int64, int64, double *);
  int64 (*matgetrank)(void*,int64*,int64);
  int64 (*matgeteig)(void*,int64,double*,double[],int64,int64[],int64*);
  int64 (*matvecvec)(void*, double[], int64,double*);
  int64 (*mataddrowmultiple)(void*,int64,double,double[],int64); /* NEEDED? */
  int64 (*matmultiply)(void*,double[],double[],int64);
  int64 (*matfactor1)(void*);
  int64 (*matfactor2)(void*,double[],int64,double[],int64,double[],int64,int64[],int64);
  int64 (*matfnorm2)(void*,int64,double*);
  int64 (*matrownz)(void*,int64,int64[],int64*,int64);
  int64 (*matnnz)(void*,int64*,int64);
  int64 (*mattest)(void*);
  int64 (*matdestroy)(void*);
  int64 (*matview)(void*);
  const char *matname;
};

#ifdef __cplusplus
extern "C" {
#endif
extern int64 DSDPGetEigs(double[],int64,double[],int64,int64[],int64, 
		       double[],int64,double[],int64,int64[],int64);
extern int64 DSDPGetEigs2(double[],int64,double[],int64,int64[],int64, 
			double[],int64,double[],int64,int64[],int64);

int64 DSDPDataMatOpsInitialize(struct  DSDPDataMat_Ops*);

#ifdef __cplusplus
}
#endif

/*
#include "dsdpdatamat.h"
*/

#endif


