#if !defined(__DSDP_DSMATRIX_H) 
#define __DSDP_DSMATRIX_H

#include "sdpconevec.h"
#include "dsdpxmat.h"
#include "dsdpbasictypes.h"
/*!
\file dsdpdsmat.h
\brief The interface between the SDPCone and the Delta S matrix
*/

/* DSDPDSMat objects are not used for much:  DS, X, eigenvalue stuff     */
/* These objects are good basically for assembling a matrix, accessing
   the data, and applying the operator to a vector                     */

/* DSDP Matrix Structure */
/*!
struct DSDPDSMat_C { void* matdata; struct DSDPDSMat_Ops* dsdpops; };

\brief Symmetric Delta S matrix for one block in the semidefinite cone.
\sa DSDPDSMat
*/
struct DSDPDSMat_C{
  void  *matdata;
  struct DSDPDSMat_Ops* dsdpops;
};

/*!
typedef struct DSDPDSMat_C DSDPDSMat;

\brief A symmetric Delta S matrix for one block in the semidefinite cone.
*/
typedef struct DSDPDSMat_C DSDPDSMat;

#ifdef __cplusplus
extern "C" {
#endif

extern int64 DSDPDSMatGetType(DSDPDSMat, int64 *);
extern int64 DSDPDSMatSetData(DSDPDSMat *, struct DSDPDSMat_Ops*,  void*);
extern int64 DSDPDSMatInitialize(DSDPDSMat*);

extern int64 DSDPDSMatZeroEntries(DSDPDSMat);
extern int64 DSDPDSMatSetArray(DSDPDSMat,DSDPVMat);
extern int64 DSDPDSMatMult(DSDPDSMat, SDPConeVec, SDPConeVec);
extern int64 DSDPDSMatVecVec(DSDPDSMat, SDPConeVec, double*);

extern int64 DSDPDSMatTest(DSDPDSMat);
extern int64 DSDPDSMatGetSize(DSDPDSMat,int64*);
extern int64 DSDPDSMatView(DSDPDSMat);
extern int64 DSDPDSMatDestroy(DSDPDSMat*);

extern int64 DSDPDSMatCheck(DSDPDSMat,SDPConeVec,SDPConeVec,DSDPVMat);
#ifdef __cplusplus
}
#endif


#endif


