#if !defined(__DSDP_DSMATRIXOPS_H) 
#define __DSDP_DSMATRIXOPS_H
#include "dsdpbasictypes.h"
/*!
\file dsdpdsmat_impl.h
\brief Structure of function pointers that each SDP Delta S matrix type 
(sparse, dense, diagonal, ...) must implement.
 */

/*!
struct DSDPDSMat_Ops

\brief Symmetric Delta S matrix for one block in the semidefinite cone.
*/
struct  DSDPDSMat_Ops{
  int64 id;
  int64 (*matzeroentries)(void*); 
  int64 (*matmult)(void*,double[],double[], int64); /* Multiply by a vector */
  int64 (*matgetsize)(void*,int64*);
  int64 (*matseturmat)(void*,double[],int64,int64); /* Set values from array */
  int64 (*matvecvec)(void*,double[],int64,double*); /* v' * DS * v */
  int64 (*mattest)(void*);
  int64 (*matview)(void*);
  int64 (*matdestroy)(void*);
  const char *matname;
};

#ifdef __cplusplus
extern "C" {
#endif
extern int64 DSDPDSMatOpsInitialize(struct  DSDPDSMat_Ops*);
#ifdef __cplusplus
}
#endif

#endif


