#if !defined(__DSDP_DUALMATRIX_H) 
#define __DSDP_DUALMATRIX_H

#include "sdpconevec.h"
#include "dsdpbasictypes.h"
#include "dsdpxmat.h"

/*!
\file dsdpdualmat.h
\brief The interface between the SDPCone and the matrix S.
 */

/*!
struct DSDPDualMat_C { void* matdata; struct DSDPDualMat_Ops* dsdpops; };

\brief Represents an S matrix for one block in the semidefinite cone.
\sa DSDPDualMat
*/
struct DSDPDualMat_C{
  void* matdata;
  struct DSDPDualMat_Ops* dsdpops;
};

/*!
\typedef struct DSDPDualMat_C   DSDPDualMat;
\brief Represents an S matrix for one block in the semidefinite cone.
*/
typedef struct DSDPDualMat_C   DSDPDualMat;

#ifdef __cplusplus
extern "C" {
#endif

extern int64 DSDPDualMatInitialize(DSDPDualMat*);
extern int64 DSDPDualMatSetData(DSDPDualMat*,struct DSDPDualMat_Ops*,void*);
extern int64 DSDPDualMatGetType(DSDPDualMat, int64 *);

extern int64 DSDPDualMatGetSize(DSDPDualMat, int64*);
extern int64 DSDPDualMatTest(DSDPDualMat);
extern int64 DSDPDualMatDestroy(DSDPDualMat *);
extern int64 DSDPDualMatView(DSDPDualMat);

extern int64 DSDPDualMatCholeskyFactor(DSDPDualMat,DSDPTruth *);
extern int64 DSDPDualMatInvert(DSDPDualMat);
extern int64 DSDPDualMatInverseAdd(DSDPDualMat,double,DSDPVMat);
extern int64 DSDPDualMatInverseMultiply(DSDPDualMat, DSDPIndex, SDPConeVec, SDPConeVec);
extern int64 DSDPDualMatCholeskySolveForward(DSDPDualMat, SDPConeVec, SDPConeVec);
extern int64 DSDPDualMatCholeskySolveBackward(DSDPDualMat, SDPConeVec, SDPConeVec);
extern int64 DSDPDualMatCholeskyForwardMultiply(DSDPDualMat, SDPConeVec, SDPConeVec);
extern int64 DSDPDualMatCholeskyBackwardMultiply(DSDPDualMat, SDPConeVec, SDPConeVec);
extern int64 DSDPDualMatLogDeterminant(DSDPDualMat, double*);
extern int64 DSDPDualMatIsFull(DSDPDualMat,DSDPTruth*);
extern int64 DSDPDualMatSetArray(DSDPDualMat,DSDPVMat);
extern int64 DSDPDualMatCheck(DSDPDualMat,SDPConeVec,SDPConeVec,DSDPIndex,DSDPVMat);
extern int64 DSDPDualMatGetArray(DSDPDualMat,double*[],int64*);

#ifdef __cplusplus
}
#endif

#endif


