#if !defined(__DSDP_DUALMATRIXOPS_H) 
#define __DSDP_DUALMATRIXOPS_H
#include "dsdpbasictypes.h"
/*!
\file dsdpdualmat_impl.h
\brief Structure of function pointers that each symmetric positive definite matrix type (dense, sparse) must implement.
*/

/*!
struct DSDPDualMat_Ops
\brief Table of function pointers that operate on the S matrix.
*/
struct  DSDPDualMat_Ops{
  int64 id;
  int64 (*matseturmat)(void*,double[],int64,int64); /* Set full array of values into matrix */
  int64 (*matgetarray)(void*,double*[],int64*);   /* Set pointers to a dense array and its dimension */
  int64 (*matcholesky)(void*,int64*);  /* Cholesky second argument is 0 for success, 1 otherwise */
  int64 (*matsolveforward)(void*,double[],double[],int64); /* Called after Cholesky */
  int64 (*matsolvebackward)(void*,double[],double[],int64); /* Called after Cholesky */
  int64 (*matinvert)(void*); /* Called after Cholesky factorization */
  int64 (*matinverseadd)(void*,double,double[],int64,int64); /* Add multiple of the inverse to array */
  int64 (*matinversemultiply)(void*,int64[],int64,double[],double[],int64); /* Called after invert */
  int64 (*matforwardmultiply)(void*,double[],double[],int64); /* Called after invert */
  int64 (*matbackwardmultiply)(void*,double[],double[],int64); /* Called after invert */
  int64 (*matlogdet)(void*,double*); /* Called after Cholesky */
  int64 (*matfull)(void*,int64*); /* Is fully dense or not? */
  int64 (*mattest)(void*);
  int64 (*matgetsize)(void*,int64*);
  int64 (*matdestroy)(void*);
  int64 (*matview)(void*);
  const char *matname;
};

#ifdef __cplusplus
extern "C" {
#endif
extern int64 DSDPDualMatOpsInitialize(struct  DSDPDualMat_Ops*);
#ifdef __cplusplus
}
#endif

#endif


