#ifndef __TAO_DSDPSDP_H
#define __TAO_DSDPSDP_H
/*! 
\file dsdpsdp.h
\brief Internal SDPCone data structures and routines
 */

#include "dsdpschurmat.h"
#include "dsdpbasictypes.h"
#include "dsdpvec.h"

#include "sdpconevec.h"
#include "dsdpdatamat.h"
#include "dsdpdualmat.h"
#include "dsdpxmat.h"
#include "dsdpdsmat.h"
#include "dsdplanczos.h"

typedef enum { SDPCONEEXIST=1, SDPCONESETUP1=2 } SDPConeStatus;

/*!
struct DSDPDataTranspose
\brief Internal structure for transpose of data.
*/
typedef struct{
  int64 m;
  int64 *nnzblocks;
  int64 **nzblocks;
  int64 *ttnzmat;
  int64 **nnzmats;
  int64 **idA;
  int64 *idAP;
} DSDPDataTranspose;

/*!
struct SDPBlockData
\brief Internal structure for data in one block of semidefintie.
*/
typedef struct{
  int64 maxnnzmats;
  int64 nnzmats;
  int64 *nzmat;
  DSDPDataMat *A;
  double r;        /* Multiple of Identity added to S to make it psd */
  double scl;
} DSDPBlockData;

/*!
struct SDPBlk
\brief Internal structure for block of semidefinite cone.
*/
typedef struct{

  DSDPBlockData ADATA;
  DSDPLanczosStepLength Lanczos;    /* For Lanczos steplength routine */

  int64 n;                                                /* Dimensions */
  double gammamu;          /* Scale Barrier, used only by user option */
  double bmu;                                    /* For LMI, not used */
  char format;       /* Packed Symmetric, Full Symmetric, Lower,Upper */
  int64   nnz;
  SDPConeStatus status;

  SDPConeVec W;
  SDPConeVec W2;
  DSDPIndex IS;

  DSDPDualMat S;      /* Dual variable matrices */
  DSDPDualMat SS;     /* Compute primal variable matrices */
  DSDPDSMat DS;       /* Dual variable step matrices */
  DSDPVMat T;         /* Work Array and Primal variable matrice X   */

} SDPblk;

/*!
struct SDPCone_C 
\brief Internal structure for semidefinite cone.
\sa SDPCone
*/
struct SDPCone_C {
  int64 keyid;

  /* Dimensions */
  int64 m, nn;

  /* Data in block format */
  int64 nblocks;
  SDPblk *blk;

  /* Transpose of Data */
  DSDPDataTranspose ATR;

  /* Work space */
  DSDPVec Work, Work2;

  /* Current Solution */
  DSDPVec YY,YX,DYX;
  double xmakermu;

  int64 optype;
  DSDP dsdp;
};

#define SDPCONEKEY  5438
#define SDPConeValid(a) {if (!(a)||((a)->keyid!=SDPCONEKEY)){ DSDPSETERR(101,"DSDPERROR: Invalid SDPCone object\n");}}

#define DSDPCHKBLOCKERR(a,b);  { if (b){ DSDPSETERR1(b,"Block Number: %d,\n",a);} }
#define DSDPCHKVARERR(a,b);  { if (b){ DSDPSETERR1(b,"Variable Number: %d,\n",a);} }

extern int64 DSDPSetDataMatZero(DSDPDataMat*);

#include "dsdp5.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Operations on the Data */
extern int64 DSDPBlockDataInitialize(DSDPBlockData*);
extern int64 DSDPBlockDataAllocate(DSDPBlockData*, int64);
extern int64 DSDPBlockAddDataMatrix(DSDPBlockData*,int64, struct DSDPDataMat_Ops*, void*);
extern int64 DSDPBlockSetDataMatrix(DSDPBlockData*,int64, struct DSDPDataMat_Ops*, void*);
extern int64 DSDPBlockRemoveDataMatrix(DSDPBlockData*,int64);
extern int64 DSDPBlockDataMarkNonzeroMatrices(DSDPBlockData*,int64*);
extern int64 DSDPBlockDataRowSparsity(DSDPBlockData*,int64,int64[],int64[],int64);
extern int64 DSDPBlockASum(DSDPBlockData*,double,DSDPVec,DSDPVMat);
extern int64 DSDPBlockADot(DSDPBlockData*,double,DSDPVec,DSDPVMat,DSDPVec);
extern int64 DSDPBlockvAv(DSDPBlockData*,double,DSDPVec,SDPConeVec,DSDPVec);
extern int64 DSDPBlockDataDestroy(DSDPBlockData*);
extern int64 DSDPBlockCheck(DSDPBlockData*,SDPConeVec,DSDPVMat);
extern int64 DSDPBlockANorm2(DSDPBlockData*, DSDPVec, int64);
extern int64 DSDPBlockView(DSDPBlockData*);
extern int64 DSDPBlockView2(DSDPBlockData*);
extern int64 DSDPBlockCountNonzeroMatrices(DSDPBlockData *,int64*);
extern int64 DSDPBlockGetMatrix(DSDPBlockData*,int64,int64*,double*,DSDPDataMat*);
extern int64 DSDPBlockFactorData(DSDPBlockData*,DSDPVMat,SDPConeVec);
extern int64 DSDPBlockTakeDownData(DSDPBlockData*);
extern int64 DSDPBlockDataRank(DSDPBlockData*,int64*,int64);

extern int64 DSDPBlockTakeDown(SDPblk*);
extern int64 DSDPBlockInitialize(SDPblk*);

extern int64 DSDPBlockEventInitialize(void);
extern int64 DSDPBlockEventZero(void);

extern int64 DSDPDataMatCheck(DSDPDataMat,SDPConeVec,DSDPIndex,DSDPVMat);

/* Operations on the Transpose of the Data */
extern int64 DSDPDataTransposeInitialize(DSDPDataTranspose*);
extern int64 DSDPDataTransposeTakeDown(DSDPDataTranspose*);
extern int64 DSDPDataTransposeSetup(DSDPDataTranspose*,SDPblk*,int64,int64);

extern int64 DSDPUseDefaultDualMatrix(SDPCone);

extern int64 SDPConeSetup(SDPCone,DSDPVec);
extern int64 SDPConeSetup2(SDPCone,DSDPVec,DSDPSchurMat);
extern int64 SDPConeComputeHessian(SDPCone,double,DSDPSchurMat,DSDPVec,DSDPVec);
extern int64 SDPConeMultiply(SDPCone,int64,double,DSDPVec,DSDPVec,DSDPVec);
extern int64 SDPConeComputeRHS(SDPCone, int64, double, DSDPVec, DSDPVec,DSDPVec);
extern int64 SDPConeComputeXX(SDPCone,int64,DSDPVec,double,DSDPDualMat, DSDPVMat);
extern int64 SDPConeDestroy(SDPCone);

extern int64 SDPConeCheckJ(SDPCone,int64);
extern int64 SDPConeCheckN(SDPCone,int64, int64);
extern int64 SDPConeCheckM(SDPCone,int64);
extern int64 SDPConeCheckStorageFormat(SDPCone,int64, char);


extern int64 SDPConeComputeSS(SDPCone, int64, DSDPVec, DSDPVMat);
extern int64 SDPConeComputeXDot(SDPCone,int64,DSDPVec,DSDPVMat,DSDPVec,double*,double*, double *);
extern int64 SDPConeComputeX3(SDPCone,int64,double,DSDPVec,DSDPVec,DSDPVMat);

/* extern int64 DSDPPrintSDPA(TAO_DSDP *); */
extern int64 DSDPMakeVMat(char, int64, DSDPVMat*);
extern int64 DSDPMakeVMatWithArray(char, double[], int64, int64, DSDPVMat*);

extern int64 DSDPSetDualMatrix(SDPCone sdpcone,int64 (*createdualmatrix)(DSDPBlockData*,DSDPVec,DSDPVMat,DSDPVec,DSDPVec,DSDPDualMat*,DSDPDualMat*,DSDPDSMat*,void*),void*);

extern int64 DSDPAddSDP(DSDP,SDPCone);
#ifdef __cplusplus
}
#endif

extern int64 SDPConeSetRIdentity(SDPCone,int64,int64,double);
extern int64 DSDPDualMatEventInitialize(void);
extern int64 DSDPVMatEventInitialize(void);
extern int64 DSDPDualMatEventZero(void);
extern int64 DSDPVMatEventZero(void);

#endif
