#if !defined(__DSDP_VSYMMETRICMATRIX_H) 
#define __DSDP_VSYMMETRICMATRIX_H

/*! 
\file dsdpxmat.h
\brief The interface between the SDPCone and the dense matrix array.
 */
#include "sdpconevec.h"
#include "dsdpbasictypes.h"
/* DSDP V Matrix Structure */
/*!
struct DSDPVMat_C { void* matdata; struct DSDPVMat_Ops* dsdpops; };

\brief Dense symmetric matrix for one block in the semidefinite cone.
\sa DSDPVMat
*/
struct DSDPVMat_C{
  void  *matdata;
  struct DSDPVMat_Ops* dsdpops;
};

/*!
\typedef struct DSDPVMat_C   DSDPVMat;
\brief Represents a dense symmetric matrix for one block in the semidefinite cone.
*/
typedef struct DSDPVMat_C DSDPVMat;

#ifdef __cplusplus
extern "C" {
#endif

extern int64 DSDPVMatGetType(DSDPVMat, int64 *);
extern int64 DSDPVMatSetData(DSDPVMat *, struct DSDPVMat_Ops*,  void*);
extern int64 DSDPVMatInitialize(DSDPVMat*);

extern int64 DSDPVMatTest(DSDPVMat);
extern int64 DSDPVMatGetSize(DSDPVMat,int64*);
extern int64 DSDPVMatView(DSDPVMat);
extern int64 DSDPVMatDestroy(DSDPVMat*);

extern int64 DSDPVMatExist(DSDPVMat,int64*);
extern int64 DSDPVMatZeroEntries(DSDPVMat);
extern int64 DSDPVMatAddOuterProduct(DSDPVMat, double, SDPConeVec);
extern int64 DSDPVMatMult(DSDPVMat,SDPConeVec,SDPConeVec);
extern int64 DSDPVMatScaleDiagonal(DSDPVMat,double);
extern int64 DSDPVMatShiftDiagonal(DSDPVMat,double);
extern int64 DSDPVMatNormF2(DSDPVMat, double*);
extern int64 DSDPVMatGetArray(DSDPVMat,double**,int64*);
extern int64 DSDPVMatRestoreArray(DSDPVMat,double**,int64*);
extern int64 DSDPVMatMinEigenvalue(DSDPVMat,SDPConeVec,SDPConeVec,double*);
extern int64 DSDPVMatCheck(DSDPVMat, SDPConeVec, SDPConeVec);

#ifdef __cplusplus
}
#endif

#endif


