#if !defined(__SDPCONE_VECTORS_H) 
#define __SDPCONE_VECTORS_H
/*!
\file sdpconevec.h
\brief Each block of the SDPCone has two vectors of appropriate size.
*/
#include <math.h>
#include "dsdpbasictypes.h"
/*!
struct SDPConeVec_C
\brief Vector whose length corresponds to dimension of a block in a cone.
\sa SDPConeVec
*/
struct  SDPConeVec_C{
  int64    dim;
  double *val;
};

typedef struct {
  int64 *indx;
} DSDPIndex;

/*!
\typedef struct SDPConeVec_C   SDPConeVec;
\brief SDPConeVec is a vector with the dimension of the block in the SDP cone.
*/
typedef struct SDPConeVec_C SDPConeVec;

#ifdef __cplusplus
extern "C" {
#endif

extern int64 SDPConeVecCreate(int64,SDPConeVec *);
extern int64 SDPConeVecDuplicate(SDPConeVec,SDPConeVec *);
extern int64 SDPConeVecDot(SDPConeVec, SDPConeVec, double *);
extern int64 SDPConeVecView( SDPConeVec);
extern int64 SDPConeVecDestroy(SDPConeVec*);
#define SDPConeVecCreateWArray(a,b,c)        0;{ (*(a)).val=(b); (*(a)).dim=(c);}

extern int64 SDPConeVecSet(double, SDPConeVec );
extern int64 SDPConeVecZero(SDPConeVec );
extern int64 SDPConeVecNormalize(SDPConeVec );
extern int64 SDPConeVecAXPY(double,  SDPConeVec,  SDPConeVec);
extern int64 SDPConeVecNorm2( SDPConeVec, double *);
extern int64 SDPConeVecCopy( SDPConeVec,  SDPConeVec);
extern int64 SDPConeVecScale(double, SDPConeVec);

#define SDPConeVecGetArray(a,b)              0;{ *(b)=((a).val); }
#define SDPConeVecRestoreArray(a,b)          0;{ *(b)=0;}
#define SDPConeVecGetSize(a,b)               0;{ *(b)=((a).dim); }

/*!
typedef struct { int64 *indx; } DSDPIndex;

\brief Identifies sparsity in SDPConeVec.
*/

extern int64 DSDPIndexInitialize(DSDPIndex*);
extern int64 DSDPIndexDestroy(DSDPIndex*);
extern int64 DSDPIndexSetBasis(DSDPIndex, int64);
extern int64 DSDPIndexCreate(int64,DSDPIndex*);
extern int64 DSDPIndexView(DSDPIndex);
#ifdef __cplusplus
}
#endif


#endif
