#ifndef __DSDPCONEOPERATIONS_H
#define __DSDPCONEOPERATIONS_H

/*!
\file dsdpcone.h
\brief The public interface between the cones and the solver.
*/

#include "dsdpbasictypes.h"
#include "dsdpvec.h"
#include "dsdpschurmat.h"


/*!
\typedef struct DSDPCone DSDPCone;

\brief This object holds the data of a SDP, LP, or other cone.  Its
structure is opaque to the DSDP Solver, but it must implement 
the interface below and provide a structure of function pointers.

*/
struct DSDPCone_C{
  void* conedata;
  struct DSDPCone_Ops* dsdpops;
};

typedef struct DSDPCone_C   DSDPCone;

#ifdef __cplusplus
extern "C" {
#endif

extern int64 DSDPConeSetUp(DSDPCone,DSDPVec);
extern int64 DSDPConeSetUp2(DSDPCone, DSDPVec, DSDPSchurMat);
extern int64 DSDPConeGetDimension(DSDPCone,double*);
extern int64 DSDPConeSparsityInSchurMat(DSDPCone, int64,int64[],int64);
extern int64 DSDPConeComputeS(DSDPCone,DSDPVec,DSDPDualFactorMatrix,DSDPTruth *);
extern int64 DSDPConeInvertS(DSDPCone);
extern int64 DSDPConeComputeHessian(DSDPCone,double,DSDPSchurMat,DSDPVec,DSDPVec);
extern int64 DSDPConeMultiplyAdd(DSDPCone,double,DSDPVec,DSDPVec,DSDPVec);
extern int64 DSDPConeComputeRHS(DSDPCone,double,DSDPVec,DSDPVec,DSDPVec);
extern int64 DSDPConeComputeMaxStepLength(DSDPCone,DSDPVec,DSDPDualFactorMatrix,double*);
extern int64 DSDPConeComputeLogSDeterminant(DSDPCone,double*,double*);
extern int64 DSDPConeComputeX(DSDPCone,double,DSDPVec,DSDPVec,DSDPVec,double*);
extern int64 DSDPConeSetXMaker(DSDPCone,double,DSDPVec,DSDPVec);
extern int64 DSDPConeView(DSDPCone);
extern int64 DSDPConeMonitor(DSDPCone,int64);
extern int64 DSDPConeDestroy(DSDPCone*);
extern int64 DSDPConeANorm2(DSDPCone,DSDPVec);
extern int64 DSDPGetConeName(DSDPCone,char*,int64);

extern int64 DSDPConeSetData(DSDPCone*,struct DSDPCone_Ops*,void*);
extern int64 DSDPConeInitialize(DSDPCone*);

#ifdef __cplusplus
}
#endif

#endif





