#ifndef __DSDPCONE_H
#define __DSDPCONE_H

/*!
\file dsdpcone_impl.h
\brief Implementations of a cone (SDP,LP,...) must provide a structure of function pointers.
*/

#include "dsdpbasictypes.h"
#include "dsdpvec.h"
#include "dsdpschurmat.h"

struct  DSDPCone_Ops{
  int64 id;
  int64 (*conesize)(void*,double*);
  int64 (*conesetup)(void*,DSDPVec);
  int64 (*conesetup2)(void*,DSDPVec,DSDPSchurMat);
  int64 (*conecomputes)(void*,DSDPVec,DSDPDualFactorMatrix,DSDPTruth*);
  int64 (*coneinverts)(void*);
  int64 (*conelogpotential)(void*,double*,double*);
  int64 (*conesetxmaker)(void*,double,DSDPVec,DSDPVec);
  int64 (*conecomputex)(void*,double,DSDPVec,DSDPVec,DSDPVec,double*);
  int64 (*conehessian)(void*,double,DSDPSchurMat,DSDPVec,DSDPVec);
  int64 (*conehmultiplyadd)(void*,double,DSDPVec,DSDPVec,DSDPVec);
  int64 (*conerhs)(void*,double,DSDPVec,DSDPVec,DSDPVec);
  int64 (*conemaxsteplength)(void*,DSDPVec,DSDPDualFactorMatrix,double*);
  int64 (*coneanorm2)(void*,DSDPVec);
  int64 (*conesparsity)(void*,int64,int64*,int64[],int64);
  int64 (*conemonitor)(void*,int64);
  int64 (*conedestroy)(void*);
  int64 (*coneview)(void*);
  const char *name;
};

extern int64 DSDPAddCone(DSDP,struct  DSDPCone_Ops*, void*);
extern int64 DSDPConeOpsInitialize(struct  DSDPCone_Ops*);

#endif
