#if !defined(__DSDP_SCHURMATRIXOPERATIONS_H) 
#define __DSDP_SCHURMATRIXOPERATIONS_H

/*!
\file dsdpschurmat.h
\brief Methods of a Schur Matrix.
*/

#include "dsdpvec.h"
#include "dsdpbasictypes.h"
#include "dsdpcg.h"


typedef struct {
  int64 *var;
  int64 nvars;
  int64 nmaxvars;
  double *fval;
  double *fdual;
  double *xout;
} FixedVariables;

typedef struct {
  FixedVariables fv;
  DSDPVec rhs3,dy3;
  double dd,r;
  int64 m;
} DSDPSchurInfo;

/*!
struct DSDPSchurMat_C
\brief Schur complement matrix whose solution is the Newton direction.
\sa DSDPSchurMat
*/
struct DSDPSchurMat_C{
  void* data;
  struct DSDPSchurMat_Ops *dsdpops;
  DSDPSchurInfo* schur;
};

/*!
\typedef struct DSDPSchurMat_C DSDPSchurMat;

\brief This object represents the Schur Matrix. Its structure is
opaque to the DSDP solver, but it must implement the interface below
and provide a structure of function pointers.

*/
typedef struct DSDPSchurMat_C   DSDPSchurMat;


#ifdef __cplusplus
extern "C" {
#endif

extern int64 DSDPSchurMatSetData(DSDPSchurMat*,struct DSDPSchurMat_Ops*, void*);

extern int64 DSDPSchurMatInitialize(DSDPSchurMat*);
extern int64 DSDPSchurMatSetup(DSDPSchurMat, DSDPVec);
extern int64 DSDPSchurMatZeroEntries(DSDPSchurMat);
extern int64 DSDPSchurMatInParallel(DSDPSchurMat, DSDPTruth*);
extern int64 DSDPSchurMatShiftDiagonal(DSDPSchurMat, double);
extern int64 DSDPSchurMatAssemble(DSDPSchurMat);
extern int64 DSDPSchurMatMultiply(DSDPSchurMat, DSDPVec, DSDPVec);
extern int64 DSDPSchurMatMultR(DSDPSchurMat, DSDPVec, DSDPVec);
extern int64 DSDPSchurMatReducePVec(DSDPSchurMat, DSDPVec);
extern int64 DSDPSchurMatFactor(DSDPSchurMat,DSDPTruth*);
extern int64 DSDPSchurMatSolve(DSDPSchurMat, DSDPVec, DSDPVec);
extern int64 DSDPSchurMatDestroy(DSDPSchurMat*);
extern int64 DSDPSchurMatView(DSDPSchurMat);
extern int64 DSDPSchurMatSetR(DSDPSchurMat, double);

extern int64 DSDPSchurMatRowColumnScaling(DSDPSchurMat,int64, DSDPVec,int64*);
extern int64 DSDPSchurMatAddRow(DSDPSchurMat, int64, double, DSDPVec);

extern int64 DSDPSchurMatVariableCompute(DSDPSchurMat, int64, double*);
extern int64 DSDPSchurMatVariableComputeC(DSDPSchurMat, double*);
extern int64 DSDPSchurMatVariableComputeR(DSDPSchurMat, double*);
extern int64 DSDPSchurMatAddDiagonalElement(DSDPSchurMat, int64, double);
extern int64 DSDPSchurMatAddC(DSDPSchurMat,int64,double);
extern int64 DSDPSchurMatAddR(DSDPSchurMat,int64,double);

extern int64 DSDPSchurMatDiagonalScaling(DSDPSchurMat, DSDPVec);
extern int64 DSDPSchurMatAddDiagonal(DSDPSchurMat, DSDPVec);

extern int64 DSDPSchurMatRowScaling(DSDPSchurMat, DSDPVec);

extern int64 DSDPZeroFixedVariables( DSDPSchurMat, DSDPVec);
extern int64 DSDPApplyFixedVariables( DSDPSchurMat, DSDPVec);
extern int64 DSDPIsFixed( DSDPSchurMat, int64, DSDPTruth*);
extern int64 DSDPInitializeFixedVariable( FixedVariables *);
extern int64 DSDPAddFixedVariable( DSDPSchurMat, int64, double);

#ifdef __cplusplus
}
#endif

#endif


