#if !defined(__DSDP_VECTORS_H) 
#define __DSDP_VECTORS_H

#include <math.h>
#include "dsdpbasictypes.h"
/*! 
\file dsdpvec.h
\brief Vector operations used by the solver
*/
/* Define DSDP Vector Structure */

/*!
\typedef struct DSDPVec_C DSDPVec;

\brief This object hold m+2 variables: a scaling of C, the y
variables, and r.  

Unscaled, this vector is \f$ [ -1 \ y_1 \ \ldots \ y_m \ r ] \f$.

*/
struct  DSDPVec_C{
  int64    dim;
  double *val;
};

typedef struct DSDPVec_C DSDPVec;

#define DSDPVecGetArray(a,b)              0;{ *(b)=((a).val); }
#define DSDPVecRestoreArray(a,b)          0;{ *(b)=0; }
#define DSDPVecGetSize(a,b)               0;{ *(b)=((a).dim); }
#define DSDPVecAddElement(a,b,c)          0;{ if (c){((a).val[b])+=(c);}  }
#define DSDPVecSetElement(a,b,c)          0;{  {((a).val[b])=(c); } }
#define DSDPVecGetElement(a,b,c)          0;{ *(c)=((a).val[b]); }
#define DSDPVecSetR(a,b)                  0;{  {((a).val[(a).dim-1])=(b); } }
#define DSDPVecAddR(a,b)                  0;{  if(b){((a).val[(a).dim-1])+=(b); } }
#define DSDPVecGetR(a,b)                  0;{ *(b)=((a).val[(a).dim-1]); }
#define DSDPVecSetC(a,b)                  0;{  {((a).val[0])=(b); } }
#define DSDPVecAddC(a,b)                  0;{  if(b){((a).val[0])+=(b); } }
#define DSDPVecGetC(a,b)                  0;{ *(b)=((a).val[0]); }
#define DSDPVecCreateWArray(a,b,c)        0;{ (*(a)).val=(b); (*(a)).dim=(c);}
/*
extern int64 DSDPVecGetArray(DSDPVec, double **);
extern int64 DSDPVecRestoreArray(DSDPVec, double **);
extern int64 DSDPVecGetSize(DSDPVec, int64 *);
extern int64 DSDPVecAddElement(DSDPVec, int64, double);
extern int64 DSDPVecSetElement(DSDPVec, int64, double);
extern int64 DSDPVecGetElement(DSDPVec, int64, double*);
extern int64 DSDPVecCreateWArray(DSDPVec*, double*, int64);
*/
#ifdef __cplusplus
extern "C" {
#endif

extern int64 DSDPVecCreateSeq(int64,DSDPVec *);
extern int64 DSDPVecDuplicate(DSDPVec,DSDPVec *);
extern int64 DSDPVecSet(double, DSDPVec );
extern int64 DSDPVecISet(int64*,DSDPVec);
extern int64 DSDPVecZero(DSDPVec );
extern int64 DSDPVecNormalize(DSDPVec );
extern int64 DSDPVecSetValue(DSDPVec,int64,double);
extern int64 DSDPVecSetBasis(DSDPVec,int64);
extern int64 DSDPVecCopy( DSDPVec,  DSDPVec);
extern int64 DSDPVecScale(double, DSDPVec);
extern int64 DSDPVecScaleCopy(DSDPVec,  double, DSDPVec);
extern int64 DSDPVecAXPY(double,  DSDPVec,  DSDPVec);
extern int64 DSDPVecAYPX(double,  DSDPVec,  DSDPVec);
extern int64 DSDPVecWAXPY(DSDPVec,double,DSDPVec,DSDPVec);
extern int64 DSDPVecWAXPBY(DSDPVec,double,DSDPVec,double,DSDPVec);
extern int64 DSDPVecPointwiseMin( DSDPVec, DSDPVec, DSDPVec);
extern int64 DSDPVecPointwiseMax( DSDPVec, DSDPVec, DSDPVec);
extern int64 DSDPVecPointwiseMult( DSDPVec, DSDPVec, DSDPVec);
extern int64 DSDPVecPointwiseDivide( DSDPVec, DSDPVec, DSDPVec);
extern int64 DSDPVecReciprocalSqrt(DSDPVec);
extern int64 DSDPVecDot(DSDPVec, DSDPVec, double *);
extern int64 DSDPVecSum( DSDPVec, double *);
extern int64 DSDPVecNorm1( DSDPVec, double *);
extern int64 DSDPVecNorm2( DSDPVec, double *);
extern int64 DSDPVecNorm22( DSDPVec, double *);
extern int64 DSDPVecNormInfinity( DSDPVec, double *);
extern int64 DSDPVecAbsoluteValue( DSDPVec);
extern int64 DSDPVecShift(double, DSDPVec);
extern int64 DSDPVecView( DSDPVec);
extern int64 DSDPVecDestroy(DSDPVec*);

#ifdef __cplusplus
}
#endif

#endif

