#include "dsdpbasictypes.h"

typedef struct {
  int64    rc;
  const double *val;
  int64    n;
  double x,y;
} rcmat;

#include "dsdpdatamat_impl.h"
#include "dsdpsys.h"
/*!
\file drowcol.c
\brief DSDPDataMat object such that A(i,j) is nonzero only if
i or j equals some integer k. Not completed.
*/

static int64 RCMatDestroy(void*);
static int64 RCMatView(void*);
static int64 RCMatVecVec(void*, double[], int64, double *);
static int64 RCMatDot(void*, double[], int64, int64, double *);
static int64 RCMatGetRank(void*, int64*, int64);
static int64 RCMatFactor(void*);
static int64 RCMatGetEig(void*, int64, double*, double[], int64, int64[], int64*);
static int64 RCMatAddRowMultiple(void*, int64, double, double[], int64);
static int64 RCMatAddMultiple(void*, double, double[], int64, int64);
static int64 RCMatGetRowNnz(void*, int64, int64[], int64*, int64);

static int64 RCMatCreate(int64 n,int64 rowcol, const double v[], rcmat**M){
  rcmat *M16;
  M16=(rcmat*) malloc(1*sizeof(rcmat));
  M16->val=v;
  M16->rc=rowcol;
  M16->n=n;
  *M=M16;
  return 0;
}

static int64 RCMatDestroy(void* AA){
  free(AA);
  return 0;
}
static int64 RCMatVecVec(void* A, double x[], int64 n, double *v){
  rcmat*RC=(rcmat*)A;
  int64 i;
  double vv=0;
  const double *val=RC->val;
  for (i=0;i<n;i++){ vv+=val[i]*x[i];}
  *v=2*vv*x[RC->rc];
  return 0;
}
static int64 RCMatDot(void* A, double x[], int64 nn, int64 n1, double *v){
  rcmat*RC=(rcmat*)A;
  int64 i,k=0,rc=RC->rc,n=RC->n;
  double vv=0;
  const double *val=RC->val;
  k=rc*(rc+1)/2;
  for (i=0;i<=rc;i++,k++){ vv+=v[k]*val[i]; }
  for (i=rc+1;i<n;i++,k+=i){ vv+=v[k+rc]*val[i];}
  *v=2*vv;
  return 0;
}
static int64 RCMatView(void* A){
  rcmat*RC=(rcmat*)A;
  int64 i;
  printf("Row Col %lld\n",RC->rc);
  for (i=0;i<RC->n;i++){
    printf("%4.4e ",RC->val[i]);
  }
  printf("\n");
  return 0;
}

static int64 RCMatFactor(void* A){
  /* Must compute x,y such that eigs are orthogonal */
  rcmat*RC=(rcmat*)A;
  int64 i,rc=RC->rc;
  double vnorm2=0;
  const double *val=RC->val;
  for (i=0;i<RC->n;i++){ vnorm2+=val[i]*val[i];} 
  vnorm2=sqrt(vnorm2);
  if (val[rc]>=0){
    RC->y=-sqrt(vnorm2);
    RC->x=sqrt(2*val[rc]+RC->y*RC->y);
  } else {
    RC->x=sqrt(vnorm2);
    RC->y=-sqrt(-2*val[rc]+RC->x*RC->x);
  }
  return 0;
}
static int64 RCMatGetRank(void *A, int64*rank, int64 n){
  *rank=2;
  return 0;
}

static int64 RCMatGetEig(void*A, int64 neig, double *eig, double v[], int64 n,int64 spind[], int64 *nind){
  rcmat*RC=(rcmat*)A;
  int64 i,rc=RC->rc;
  double x=RC->x,y=RC->y,xmy=x-y;
  const double *val=RC->val;
  if (neig==0){
    for (i=0;i<n;i++){spind[i]=i;}
    for (i=0;i<n;i++){v[i]=val[i]/xmy;}
    v[rc]=RC->x;
    *nind=n;
    *eig=1.0;
  } else if (neig==1){
    for (i=0;i<n;i++){spind[i]=i;}
    for (i=0;i<n;i++){v[i]=val[i]/xmy;}
    v[rc]=RC->y;
    *nind=n;
    *eig=-1.0;
  } else { *eig=0;*nind=0;}
  return 0;
}

static int64 RCMatGetRowNnz(void*A, int64 nrow, int64 nz[], int64 *nnzz, int64 n){
  rcmat*RC=(rcmat*)A;
  int64 i;
  *nnzz=1;
  if (nrow==RC->rc){for (i=0;i<n;i++){nz[i]++;}*nnzz=n;}
  nz[nrow]++;
  return 0;
}

static int64 RCMatAddRowMultiple(void*A, int64 nrow, double dd, double rrow[], int64 n){
  rcmat*RC=(rcmat*)A;
  int64 i;
  if (nrow==RC->rc){ 
    for (i=0;i<n;i++){rrow[i]+=dd*RC->val[i];}
  }
  rrow[nrow]+=dd*RC->val[nrow];
  return 0;
}
static int64 RCMatAddMultiple(void*A, double dd, double vv[], int64 nn, int64 n1){
  rcmat*RC=(rcmat*)A;
  int64 i,rc=RC->rc,k=rc*(rc+1)/2,n=RC->n;
  const double *val=RC->val;
  for (i=0;i<=rc;i++,k++){
    vv[k]+=dd*val[i];
  }
  for (i=rc+1;i<n;i++,k+=i){
    vv[k+rc]+=dd*val[i];
  }
  return 0;
}
static int64 RCMatFNorm(void*A, int64 n, double *fnorm){
  rcmat*RC=(rcmat*)A;
  int64 i,rc=RC->rc;
  double ff=0;
  const double *val=RC->val;
  for (i=0;i<n;i++){
    ff+=val[i]*val[i];
  }
  ff*=2;
  ff+=2*val[rc]*val[rc];
  *fnorm=ff;
  return 0;
}
static int64 RCMatCountNonzeros(void*A, int64 *nnz, int64 n){
  *nnz=2*n-1;
  return 0;
}
static struct  DSDPDataMat_Ops rcmatops;
static const char *datamatname="One Row and Column matrix";
static int64 RCMatOperationsInitialize(struct  DSDPDataMat_Ops* rcmatoperator){
  int64 info;
  if (rcmatoperator==NULL) return 0;
  info=DSDPDataMatOpsInitialize(rcmatoperator); if (info){ return info;}
  rcmatoperator->matfactor1=RCMatFactor;
  rcmatoperator->matgetrank=RCMatGetRank;
  rcmatoperator->matgeteig=RCMatGetEig;
  rcmatoperator->matvecvec=RCMatVecVec;
  rcmatoperator->matrownz=RCMatGetRowNnz;
  rcmatoperator->matdot=RCMatDot;
  rcmatoperator->matfnorm2=RCMatFNorm;
  rcmatoperator->matnnz=RCMatCountNonzeros;
  rcmatoperator->mataddrowmultiple=RCMatAddRowMultiple;
  rcmatoperator->mataddallmultiple=RCMatAddMultiple;
  rcmatoperator->matdestroy=RCMatDestroy;
  rcmatoperator->matview=RCMatView;
  rcmatoperator->matname=datamatname;
  rcmatoperator->id=27;
  return 0;
}


#undef __FUNCT__
#define __FUNCT__ "DSDPGetRCMat"
int64 DSDPGetRCMat(int64 n,const double val[],int64 rowcol, struct  DSDPDataMat_Ops**sops, void**smat){
  rcmat *AA;
  int64 info;
  DSDPFunctionBegin;
  info=RCMatCreate(n,rowcol,val,&AA);DSDPCHKERR(info);
  info=RCMatOperationsInitialize(&rcmatops); DSDPCHKERR(info);
  if (sops){*sops=&rcmatops;}
  if (smat){*smat=(void*)AA;}
  DSDPFunctionReturn(0);
}

