#include "dsdpdatamat_impl.h"
#include "dsdpsys.h"
/*!
\file zeromat.c
\brief DSDPDataMat object with all zero entries.
*/
static int64 ZDestroy(void*);
static int64 ZView(void*);
static int64 ZVecVec(void*, double[], int64, double *);
static int64 ZDot(void*, double[], int64, int64,double *);
static int64 ZGetRank(void*, int64*,int64);
static int64 ZFactor(void*);
static int64 ZGetEig(void*, int64, double*, double[], int64,int64[],int64*);
static int64 ZAddRowMultiple(void*, int64, double, double[], int64);
static int64 ZAddMultiple(void*, double, double[], int64,int64);
static int64 ZRowNnz(void*, int64, int64[], int64*, int64);

static struct  DSDPDataMat_Ops zeromatops;
static int64 ZeroMatopsInitialize(struct DSDPDataMat_Ops*);

int64 DSDPGetZeroDataMatOps(struct DSDPDataMat_Ops** zops){
  int64 info;
  info=ZeroMatopsInitialize(&zeromatops); if (info){return info;}
  if (zops){*zops=&zeromatops;}
  return info;
}

static int64 ZFactor(void *A){
  return 0;
}

static int64 ZGetRank(void*A,int64*rank,int64 n){
  *rank=0;
  return 0;
}

static int64 ZGetEig(void*A,int64 neig, double *eig, double v[], int64 n,int64 indx[],int64*nind){
  *eig=0.0;
  *nind=0;
  return 0;
}

static int64 ZDot(void*A, double x[], int64 nn, int64 n, double *sum){
  *sum=0.0;
  return 0;
}

static int64 ZVecVec(void*A, double x[], int64 n, double *sum){
  *sum=0.0;
  return 0;
}

static int64 ZAddMultiple(void*A, double dd, double row[], int64 nn, int64 n){
  return 0;
}

static int64 ZAddRowMultiple(void*A, int64 nrow, double dd, double row[], int64 n){
  return 0;
}

static int64 ZRowNnz(void*A, int64 row, int64 nz[], int64 *nnz, int64 n){
  *nnz=0;
  return 0;
}

static int64 ZDestroy(void*A){
  return 0;
}

static int64 ZNorm2(void*A,int64 n,double *v){
  *v=0;
  return 0;
}

static int64 ZView(void*A){
  printf("All zeros\n");
  return 0;
}

static const char* datamatname="MATRIX OF ZEROS";

static int64 ZeroMatopsInitialize(struct  DSDPDataMat_Ops* sops){
  int64 info;
  if (sops==NULL) return 0;
  info=DSDPDataMatOpsInitialize(sops); if (info){ return info;}
  sops->matfactor1=ZFactor;
  sops->matgetrank=ZGetRank;
  sops->matgeteig=ZGetEig;
  sops->matvecvec=ZVecVec;
  sops->matdot=ZDot;
  sops->matfnorm2=ZNorm2;
  sops->matrownz=ZRowNnz;
  sops->mataddrowmultiple=ZAddRowMultiple;
  sops->mataddallmultiple=ZAddMultiple;
  sops->matdestroy=ZDestroy;
  sops->matview=ZView;
  sops->id=10; 
  sops->matname=datamatname;
  return 0;
}



